@@ -4,7 +4,7 @@ use strict;
 use warnings;
 
 use 5.008;
-our $VERSION = '0.0.2';
+our $VERSION = '0.0.4';
 $VERSION = eval $VERSION;
 
 
@@ -24,6 +24,7 @@ use Carp;
 use SQL::Maker;
 use DateTime;
 use Data::Dumper;
+use String::Random;
 use Class::Accessor::Lite (
     new     => 1,
     rw      => [
@@ -80,6 +81,11 @@ my %VALUE_DEF_FUNC = (
     year        => \&_val_year,
 );
 
+# When convert regex into string, some prefix and postfix is added to pattern
+# like (?^:AAAAA)
+# These are used to remove them from string converted from regex.
+my ($REGEX_TO_STRING_PREFIX, $REGEX_TO_STRING_POSTFIX) = (scalar qr/AAAAA/) =~ /^(.*)AAAAA(.*)$/;
+
 
 =head1 NAME
 
@@ -95,19 +101,19 @@ This documentation refers to Data::HandyGen::mysql version 0.0.2
 
     use DBI;
     use Data::HandyGen::mysql;
-       
+
     my $dbh = DBI->connect('dbi:mysql:test', 'user', 'pass');
-    
+
     my $hd = Data::HandyGen::mysql->new( fk => 1 );
     $hd->dbh($dbh);
-     
-    
+
+
     #  -- table definitions --
     #
     #  create table category (
     #      id           integer primary key,
     #      name         varchar(20) not null
-    #  ); 
+    #  );
     #
     #  create table item (
     #      id           integer primary key auto_increment,
@@ -116,16 +122,16 @@ This documentation refers to Data::HandyGen::mysql version 0.0.2
     #      price        integer not null,
     #      constraint foreign key (category_id) references category(id)
     #  );
-    
-    
+
+
     #  1.
     #  Insert one row to 'item'.
     #  'category_id', 'name' and 'price' will be random values.
     #  category_id refers to category.id, so the value will be selected one of values in category.id.
-    #  If table 'category' has no record, new record will be added to 'category'. 
-    
+    #  If table 'category' has no record, new record will be added to 'category'.
+
     my $id = $hd->insert('item');
-    
+
     #  Result example:
     #  [item]
     #           id: 1
@@ -136,17 +142,17 @@ This documentation refers to Data::HandyGen::mysql version 0.0.2
     #  [category]
     #           id: 497364651
     #         name: name_497364651
-    #    
-    
+    #
+
     print "ID: $id\n";      #  'ID: 1'
-    
-        
+
+
     #  2.
     #  Insert one row to 'item' with name = 'Banana'.
-    #  category_id and price will be random values. 
-    
+    #  category_id and price will be random values.
+
     $id = $hd->insert('item', { name => 'Banana' });  #  Maybe $id == 2
-    
+
     #  Result example:
     #  [item]
     #           id: 2
@@ -157,17 +163,17 @@ This documentation refers to Data::HandyGen::mysql version 0.0.2
     #  [category]
     #           id: 497364651
     #         name: name_497364651
-    
-     
-    #  3.      
+
+
+    #  3.
     #  Insert one row to 'item' with category_id one of 10, 20 or 30 (selected randomly).
-    #  If table 'category' has no record with id = 10, 20 nor 30, 
+    #  If table 'category' has no record with id = 10, 20 nor 30,
     #  a record having one of those ids will be generated on 'category'.
-    
+
     $hd->insert('item', { category_id => [ 10, 20, 30 ] });
-    
+
     #  Result example:
-    #  [item] 
+    #  [item]
     #           id: 3
     #  category_id: 20
     #         name: name_3
@@ -182,12 +188,12 @@ This documentation refers to Data::HandyGen::mysql version 0.0.2
     #  If you're interested also in category name, do this.
 
     $cat_id = $hd->insert('category', { name => 'Fruit' });
-    $item_id = $hd->insert('item', { category_id => $cat_id, name => 'Coconut' });    
+    $item_id = $hd->insert('item', { category_id => $cat_id, name => 'Coconut' });
+
 
-   
     #  Delete all records inserted by $hd
-    $hd->delete_all();     
-    
+    $hd->delete_all();
+
     #  ...Or retrieve all IDs for later deletion.
     my $ids = $hd->inserted();
 
@@ -199,7 +205,7 @@ This module generates test data and insert it into mysql tables. You only have t
 When we test our product, sometimes we need to create test records, but generating them is a tedious task. We should consider many constraints (not null, foreign key, etc.) and set values to many columns in many tables, even if we want to do small tests, are interested in only a few columns and don't want to care about others. Maybe this module get rid of much of those unnecessary task.
 
 
-=head1 METHODS 
+=head1 METHODS
 
 
 =head2 new(dbh => $dbh, fk => $fk)
@@ -275,6 +281,18 @@ value of 'colname' will be randomly chosen from $val1, $val2, ...
 
 verbose expression of above
 
+=item * colname => qr/$pattern/
+
+value of 'colname' is determined by $pattern.
+
+NOTE: This function uses randregex of C<String::Random>, which does not handles real regular expression.
+
+    $hd->insert('table1', { filename => qr/[0-9a-f]{8}\.jpg/ });  #  'a1b2c3d4.jpg'
+
+=item * colname => { random => qr/$pattern/ }
+
+verbose expression of above
+
 =item * colname => { range => [ $min, $max ] }
 
 value of 'colname' is determined between $min and $max ($min inclusive, $max exclusive). Can be used only for number(int, double, numeric, etc.).
@@ -283,8 +301,8 @@ value of 'colname' is determined between $min and $max ($min inclusive, $max exc
 
 value of 'colname' is determined between $start_datetime and $end_datetime ($start_datetime inclusive, $end_datetime exclusive). Can be used only for date or datetime type.
 
-    $hd->insert('table1', { 
-        purchase_datetime => { dt_range => [ '2013-07-20 12:00:00', '2013-7-21 14:00:00' ] } 
+    $hd->insert('table1', {
+        purchase_datetime => { dt_range => [ '2013-07-20 12:00:00', '2013-7-21 14:00:00' ] }
     });
 
     $hd->insert('table2', {
@@ -301,7 +319,7 @@ Returns a value of primary key. (Only when primary key exists and it contains on
 =cut
 
 #  XXX: I commented out lines below, because this function does not work properly.
-# 
+#
 #=head3 column name in other tables
 #
 #If you want to specify values of other tables (maybe referenced by foreign key), join table name and column name with dot(.)
@@ -327,7 +345,7 @@ sub process_table {
     my $dbh = $self->dbh();
 
     #  Reads an additional spec
-    $tmpl_valspec 
+    $tmpl_valspec
         and $self->_add_user_valspec($table, $tmpl_valspec);
     $self->_print_debug("tmpl_valspec : " . Dumper($self->_valspec()));
 
@@ -338,7 +356,7 @@ sub process_table {
     #  $real_id : User specified value if specified. Otherwise undef.
     my ($exp_id, $real_id) = $self->get_id($table, $tmpl_valspec);
     $self->_print_debug("id is (" . ($exp_id || '(undef)') . ", " . ($real_id || '(undef)') . ")");
-    
+
 
     #  columns to which we need to specify values.
     my @colnames = $self->get_cols_requiring_value($table, $table_def->def);
@@ -349,21 +367,21 @@ sub process_table {
     for my $col (@colnames) {
 
         my $value;
-    
+
         #  (1)Primary key, and a value is specified by user.
         if ( $table_def->is_pk($col) and defined($real_id) ) {
             $values{$col} = $real_id;
             next;
         }
 
-        my $col_def = $table_def->column_def($col) 
+        my $col_def = $table_def->column_def($col)
             or confess "No column def found. $col";
 
 
         #  (2)If $self->fk = 1 and the column is a foreign key.
         if ( $self->fk ) {
             if ( my $referenced_table_col = $table_def->is_fk($col) ) {     #  ret = { table => 'table name, column => 'column name' }
-                if ( ref $referenced_table_col eq 'HASH' ) { 
+                if ( ref $referenced_table_col eq 'HASH' ) {
                     $value = $self->determine_fk_value($table, $col, $referenced_table_col);
                 }
                 else {
@@ -376,11 +394,11 @@ sub process_table {
         if ( !defined($value) and defined( my $valspec_col = $self->_valspec()->{$table}{$col} ) ) {
             $value = $self->determine_value( $valspec_col );
         }
-        
+
         #  (3.5)If column default is available, use it.
         if ( !defined($value) and defined($col_def->column_default) ) {
             $value = $col_def->column_default;
-        }            
+        }
 
         #  (4)Otherwise, decide a value randomly.
         if ( !defined($value) ) {
@@ -393,7 +411,7 @@ sub process_table {
                 warn "Type $type for $col is not supported.";
                 next;
             }
-            
+
             $value = $self->$func($col_def, $exp_id);
             $self->_print_debug("No rule found. Generates random value.($value)");
 
@@ -417,18 +435,18 @@ sub process_table {
     if ($@) {
         confess $@
     }
-        
+
     my $inserted_id = undef;
-   
-    
+
+
     #  Handles PK value only when the table has single pk column.
     if ( @{ $table_def->pk_columns() } == 1 ) {
         $inserted_id = $real_id || $dbh->{'mysql_insertid'};
         $self->add_inserted_id($table, $inserted_id);
-   
+
         $self->_print_debug("Inserted. table = $table, id = $inserted_id");
     }
-    
+
     return $inserted_id;
 }
 
@@ -475,14 +493,23 @@ sub determine_value {
     if ( exists($valspec_col->{random}) ) {
         my $values = $valspec_col->{random};
 
-        ref $values eq 'ARRAY'
-            or confess "Value of 'random' is invalid. type = " . (ref $values);
-        scalar(@$values) > 0
-            or confess "Value of 'random' is an empty arrayref";
-
-        my $ind = rand() * scalar(@$values);
-        $value = $values->[$ind]; 
+        if (ref $values eq 'ARRAY') {
+            if (scalar(@$values) == 0) {
+                confess "Value of 'random' is an empty arrayref";
+            }
 
+            my $ind = rand() * scalar(@$values);
+            $value = $values->[$ind];
+        }
+        elsif (ref $values eq 'Regexp') {
+            my $pattern = scalar $values;
+            $pattern =~ s/^\Q$REGEX_TO_STRING_PREFIX\E//;
+            $pattern =~ s/\Q$REGEX_TO_STRING_POSTFIX\E$//;
+            $value = String::Random::random_regex($pattern);
+        }
+        else {
+            confess "Value of 'random' is invalid. type = " . (ref $values);
+        }
     }
     elsif ( exists($valspec_col->{fixval}) ) {
         my $fixval = $valspec_col->{fixval};
@@ -497,16 +524,16 @@ sub determine_value {
     }
     elsif ( exists($valspec_col->{range} ) ) {
         my $spec = $valspec_col->{range};
-        ref $spec eq 'ARRAY' and @$spec == 2 
+        ref $spec eq 'ARRAY' and @$spec == 2
             or confess "Value of 'range' must be an arrayref with (begin, end) values";
         $value = _get_random_range(@$spec);
     }
     elsif ( exists($valspec_col->{dt_range}) ) {
         my $spec = $valspec_col->{dt_range};
-        ref $spec eq 'ARRAY' and @$spec == 2 
+        ref $spec eq 'ARRAY' and @$spec == 2
             or confess "Value of 'dt_range' must be an arrayref with (start, end) values";
         $value = _get_random_dt_range(@$spec);
-    }        
+    }
 
     return $value;
 }
@@ -525,7 +552,7 @@ sub _get_random_dt_range {
 
     my $start_epoch = _get_epoch($start);
     my $end_epoch   = _get_epoch($end);
-    
+
     my $value = DateTime
                 ->from_epoch( epoch => $start_epoch + rand($end_epoch - $start_epoch) )
                 ->strftime("%Y-%m-%d %H:%M:%S");
@@ -538,7 +565,7 @@ sub _get_epoch {
     my ($timestr) = @_;
 
     #  time format is expected to 'yyyy-mm-dd hh:mm:ss'
-    my @ymdhms = split /\D/, $timestr;     
+    my @ymdhms = split /\D/, $timestr;
     my $dt = DateTime->new(
         year    => $ymdhms[0],
         month   => $ymdhms[1] || 1,
@@ -579,7 +606,7 @@ sub determine_fk_value {
     my $ref_table = $ref->{table};
     my $ref_col   = $ref->{column};
 
-    $table and $col and $ref_table and $ref_col 
+    $table and $col and $ref_table and $ref_col
         or confess "Invalid args. (requires 3 args)";
 
     $self->_print_debug("Column $col is a foreign key references $ref_table.$ref_col.");
@@ -587,14 +614,14 @@ sub determine_fk_value {
     if ( my $valspec_col = $self->_valspec()->{$table}{$col} || $self->_valspec()->{$ref_table}{$ref_col} ) {
         $self->_print_debug("Value is specified.");
 
-        # 
+        #
         #  (1)If a rule of determining the value is specified by user, apply the rule.
         #
         $value = $self->determine_value( $valspec_col );
 
-        #  If a referenced record does not exist in a referenced table, 
+        #  If a referenced record does not exist in a referenced table,
         #  insert a record having the value at first.
-        #  
+        #
         #  * I haven't thought it would be efficient to query every time which values
         #    in a given column in a referenced table exist. At first I used to believe
         #    it would be a good idea to query only for the first time, and cache those values
@@ -621,8 +648,8 @@ sub determine_fk_value {
         #  Its result is like...
         #  $ref_ids => { (id1)  => 1, (id2) => 1, ... }
         #
-        my $ref_ids = $self->_get_current_distinct_values($ref_table, $ref_col); 
-    
+        my $ref_ids = $self->_get_current_distinct_values($ref_table, $ref_col);
+
 
         #  Pick up one of referenced values randomly, if at least one record exists.
         my @_ref_ids = keys %$ref_ids;
@@ -636,7 +663,7 @@ sub determine_fk_value {
             $value = $self->process_table($ref_table);      #  ID value would be determined randomly.
             $self->_distinct_val()->{$ref_table}{$ref_col}{$value} = 1;            #  Add the ID value
             $self->_print_debug("Referenced record created. id = $value");
-            
+
         }
     }
 
@@ -649,10 +676,10 @@ sub determine_fk_value {
 #  Returns 2 values. One if exp_id(expected ID), which is used to determine column values
 #  other than primary key (for example, when expected id is 4001, values of column named 'foo'
 #  will be 'foo_4001' if possible.
-#  Another is real_id, which is a final value of ID column. It may be undef if no value is 
+#  Another is real_id, which is a final value of ID column. It may be undef if no value is
 #  specified by user.
-#  
-#  TODO: Currently it works properly only when primary key consists of one column, 
+#
+#  TODO: Currently it works properly only when primary key consists of one column,
 #  and its type is integer.
 sub get_id {
     my ($self, $table) = @_;
@@ -664,14 +691,14 @@ sub get_id {
     for my $col (@$pks) {   #  for each pk columns
 
         my $col_def = $table_def->column_def($col);
-        
+
 
         #  Verifies if PK value can be determined by the user-specified rule.
         #  If possible, $real_id will be a value determined by the rule.
-        if (    $self->_valspec()->{$table} 
+        if (    $self->_valspec()->{$table}
                 and defined( $self->_valspec()->{$table}{$col} )
                 and defined( $real_id = $self->determine_value( $self->_valspec()->{$table}{$col} ) )
-        ) 
+        )
         {
 
             #  exp_id will be the same of real_id when user-specified rule exists.
@@ -688,7 +715,7 @@ sub get_id {
                 #  If the PK has auto_increment attribute, retrieve a value from it.
                 $self->_print_debug("Column $col is an auto_increment");
                 $exp_id = $table_def->get_auto_increment_value();
-                
+
                 #  real_id won't be determined until insert operation executes, so leaves it undef.
 
             }
@@ -699,14 +726,14 @@ sub get_id {
                 my $size = $col_def->character_maximum_length;
                 my $func = $VALUE_DEF_FUNC{$type}
                     or die "Type $type for $col not supported";
-                
+
                 $exp_id = $real_id = $self->$func($col_def);
 
             }
         }
     }
 
-    return ($exp_id, $real_id);             
+    return ($exp_id, $real_id);
 }
 
 
@@ -738,14 +765,14 @@ sub get_cols_requiring_value {
             }
 
             #
-            #  I used to believe that DEFAULT value could be used if exists, so 
+            #  I used to believe that DEFAULT value could be used if exists, so
             #  I should skip the column having DEFAULT value.
             #  But I found it wouldn't work properly when the column has
-            #  foreign key constraint too, because it seemes there would be 
+            #  foreign key constraint too, because it seemes there would be
             #  no way to add a record to referenced table.
             #  So I've changed the way assuming the user rule would be specified
             #  as the DEFAULT value.
-            #  
+            #
             #  Skip only when the column isn't a foreign key and has default value.
             if ( defined($col_def->column_default) and not $table_def->is_fk($col) ) {
                 $self->_print_debug("column $col has default value and not FK, so no need to assign value");
@@ -771,7 +798,7 @@ sub get_cols_requiring_value {
 sub _table_def {
     my ($self, $table) = @_;
 
-    $self->{_table_def}{$table} 
+    $self->{_table_def}{$table}
         ||= Data::HandyGen::mysql::TableDef->new( dbh => $self->dbh, table_name => $table );
 
     return $self->{_table_def}{$table};
@@ -782,7 +809,7 @@ sub _table_def {
 #  _val_varchar($col_def, $exp_id)
 #
 #  Creates a new varchar value.
-#  
+#
 #  $col_def : ColumnDef object.
 #  $exp_id  : an expected value of primary key.
 #
@@ -806,10 +833,10 @@ sub _val_varchar {
         }
         elsif ( $pk_length == $maxlen ) {
             return $exp_id;
-        }   
+        }
     }
 
-    $maxlen > $LENGTH_LIMIT_VARCHAR 
+    $maxlen > $LENGTH_LIMIT_VARCHAR
         and $maxlen = $LENGTH_LIMIT_VARCHAR;
     $self->_print_debug("Maxlen is $maxlen");
 
@@ -852,14 +879,14 @@ sub _val_int {
 
 sub _make_float {
     my ($precision, $scale) = @_;
-    
+
     my $num = '';
     $num .= int(rand() * 10) for 1 .. $precision - $scale;
     if ( $num =~ /^0+$/ ) {
         $num = '0'
     }
     else {
-        $num =~ s/^0+//; 
+        $num =~ s/^0+//;
     }
 
     if ( $scale > 0 ) {
@@ -931,10 +958,10 @@ sub _get_current_distinct_values {
 
     my $current;
 
-    #  At first, I tried to cache distinct values, but when user delete records, 
+    #  At first, I tried to cache distinct values, but when user delete records,
     #  those cached values are incorrect, and this module has no idea
     #  which records have been already deleted.
-    #  So I decide not to cache distinct values and query them every time. 
+    #  So I decide not to cache distinct values and query them every time.
 
     #my $current = $self->_distinct_val()->{$table}{$col};
     #if ( !defined $current or keys %$current == 0 ) {
@@ -990,7 +1017,7 @@ sub _add_user_valspec {
 
 
     for my $col (keys %$table_valspec) {
-         
+
         my $_table = $table;
         my $_col   = $col;
 
@@ -998,10 +1025,10 @@ sub _add_user_valspec {
             ($_table, $_col, my @_dummy) = split '\.', $col;
 
             #  column name may include only one dot.
-            defined($_table) and length($_table) > 0 
+            defined($_table) and length($_table) > 0
             and defined($_col) and length($_col) > 0
-            and @_dummy == 0 
-                or confess "Invalid column name : $col"; 
+            and @_dummy == 0
+                or confess "Invalid column name : $col";
         }
 
         my $val = $table_valspec->{$col};
@@ -1009,14 +1036,14 @@ sub _add_user_valspec {
         #  At first, clear all values with the same key.
         delete $self->_valspec()->{$_table}{$_col};
 
-        if ( ref $val eq 'ARRAY' ) {
+        if ( ref $val eq 'ARRAY' or ref $val eq 'Regexp' ) {
             #  arrayref : select one from the list randomly.
             $self->_valspec()->{$_table}{$_col}{random} = $val;
 
         }
         elsif ( ref $val eq 'HASH' ) {
-            #  hash : 
-            #  currently { random => [ ... ] } or { fixval => $scalar } 
+            #  hash :
+            #  currently { random => [ ... ] } or { fixval => $scalar }
             #  may be specified.
             for (keys %$val) {
                 $self->_valspec()->{$_table}{$_col}{$_} = $val->{$_};
@@ -1034,12 +1061,12 @@ sub _add_user_valspec {
 
         }
         else {
-            confess "NOTREACHED";
+            confess "Invalid spec of column. Column name = [$col]";
         }
 
     }
 
-}   
+}
 
 
 =head2 inserted()
@@ -1047,7 +1074,7 @@ sub _add_user_valspec {
 Returns all primary keys of inserted records by this instance. Returned value is a hashref like this:
 
     my $ret = $hd->inserted();
-    
+
     #  $ret = {
     #    'table_name1' => [ 10, 11 ],
     #    'table_name2' => [ 100, 110, 120 ],
@@ -1176,5 +1203,4 @@ modify it under the same terms as Perl itself. See L<perlartistic>.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
